﻿using System;
using System.Net;
using System.Text;
using Newtonsoft.Json;
using RestSharp;
/* Introduction.
 * In this Tutorial we will write a simple python code that can send a message to any phone number
 * You can run this application either on Terminal or in your preferred IDE.
 * The application will receive 2 input parameters that will be asked for the user upon its execution
 * First parameter will be the Message to be sent.
 * Second parameter will be the phone that will receive the message.
 * Requirements:
 * Basic Programming experience with C#.
 * Newtonsoft JSON package
 * RestSharp package
 * Generated an app_key and app_secret through the Live Link Portal
 * For that, go to the portal https://livelink.sapmobileservices.com/documentation/
 * Then go to Getting Started >> Step 3: Create your first APP Key
 */

namespace HelloWorld
{

	static class Globals
	{
		/* Step 1
		 * SETUP ENVIROMENT VARIABLES
		 * On this step we will setup all the needed Variables to make the Live Link System Work.
		 * baseUrl -> the Live Link URL that will be used to make the calls to Live Link API
		 * appKey -> the key generated through the Live Link Portal
		 * appSecret -> the secret that you received when creating a new app_key through the Live Link Portal.
		 * defaultOrigin -> that is the origin that you want the phone number to receive messages from.
		 * All this information will be used in the next functions.
		 */
		public const String baseUrl = "https://localhost:8080/api";
		public const String appKey = "DI9eZ0xGklRxkQmE"; //Insert your generated app key 
		public const String appSecret = "alvwzdMxWrO6zWjb2JeYAEn7llpKYwIA"; //Insert your generated app secret
		public const String defaultOrigin = "3145"; //Insert your default origin number 

		/* Extra info.
		 * If you are Behind a corporate proxy, you need to set the proxy to send HTTP Calls.
		 * public static WebProxy globalProxy = new WebProxy("http://proxy", 8080);
		 */
		public const WebProxy globalProxy = null;
	}

	public class HelloWorld
	{
		/* Step 2
		 * Defining Function to retrieve the oAuth2 access_token through client_credentials
		 */
		public static String GetAccessToken(String key = Globals.appKey, String secret = Globals.appSecret)
		{
			Console.WriteLine("...... executing authorization request with key: " + key);

			String oAuthUrl = Globals.baseUrl + "/oauth/token";
			String bearer = Convert.ToBase64String(Encoding.UTF8.GetBytes(key + ":" + secret));
			RestClient client = new RestClient(oAuthUrl);
			if (Globals.globalProxy != null) {
				client.Proxy = Globals.globalProxy;
				client.Proxy.Credentials = System.Net.CredentialCache.DefaultCredentials;
			}
			RestRequest request = new RestRequest(Method.POST);
			request.AddHeader("content-type", "application/x-www-form-urlencoded");
			request.AddHeader("authorization", "Basic " + bearer);
			request.AddParameter("application/x-www-form-urlencoded", "grant_type=client_credentials", ParameterType.RequestBody);

			IRestResponse<oAuthToken> response = client.Execute<oAuthToken>(request);

			if (response.StatusCode != HttpStatusCode.OK) {
				Console.WriteLine("..... Error: " + response.StatusDescription);
				return "";
			}

			String accessToken = response.Data.access_token;
			Console.WriteLine("..... authorization request successfull access token: " + accessToken);
			return accessToken;
		}

		/* Step 3
		 * Function to send message with two mandatory parameters, message and destination.
		 * When calling this function, this will use requests library to prepare an HTTP Call
		 * This HTTP Call will have an Authorization Header with a Bearer access_token
		 * that was retrieved from the getAccessToken function.
		 */
		public static void sendSMS(String message, String destination)
		{
			Console.WriteLine("..... sending message");

			String messageUrl = Globals.baseUrl + "/v2/sms";
			Object messagePayload = new {
				message = message,
				origin = Globals.defaultOrigin,
				destination = destination
			};
			String messageData = JsonConvert.SerializeObject(messagePayload);
			String bearer = GetAccessToken();
			if (String.IsNullOrEmpty(bearer)) {
				return;
			}
			RestClient client = new RestClient(messageUrl);
			if (Globals.globalProxy != null) {
				client.Proxy = Globals.globalProxy;
				client.Proxy.Credentials = System.Net.CredentialCache.DefaultCredentials;
			}

			RestRequest request = new RestRequest(Method.POST);
			request.AddHeader("authorization", "Bearer " + bearer);
			request.AddHeader("content-type", "application/json");
			request.AddParameter("application/json", messageData, ParameterType.RequestBody);

			IRestResponse response = client.Execute(request);

			if (response.StatusCode != HttpStatusCode.OK) {
				Console.WriteLine("..... failed to send message.");
				Console.WriteLine("..... Error: " + response.ErrorMessage);
				return;
			}
			Console.WriteLine("..... Message sent successfully");
			return;
		}

		/*Step 4
		 * Now we will write a program that receives 2 inputs
		 *  First it will ask for the Message that you want to send
		 * Then it will ask for the phone number that you want to send the message to.
		 */
		public static void Main(string[] args)
		{
			Console.Clear();
			Console.Write("What do you want to say(text message): ");
			String message = Console.ReadLine();
			Console.Write("Who do you want to send to(phone number): ");
			String destination = Console.ReadLine();
			Console.WriteLine("****************************************");
			Console.WriteLine("Sending: " + message);
			Console.WriteLine("To: " + destination);
			sendSMS(message, destination);
		}
	}
}